 /***************************************************************************
 *                                                                          *
 *                               TESTLIST                                   *
 *                                                                          *
 *                       Illustrates how to use dllist                      *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/



#include <string.h>
#include "dllist.h"



#ifdef __cplusplus
extern "C"
{
#endif


typedef struct
{
   int value;
} node; /* one node */



extern int pfnCompareListNodes ( const void * pvPrimaryRecord,
                                 const void * pvSecondaryRecord )
/*
   Compares list nodes containing integers
*/
{
   const node * pPRec = ( const node *) pvPrimaryRecord;
   const node * pSRec = ( const node *) pvSecondaryRecord;
   if ( pPRec->value > pSRec->value )
      return 1;
   else if ( pPRec->value < pSRec->value )
      return -1;
   return 0;
}


extern void * pfnCreateListNode ( const void * pvData )
/*
   Create an integer list node
*/
{
   void   * pvNodeData = NULL;

   assert ( pvData != NULL );
   if ( NULL == ( pvNodeData = calloc ( 1, sizeof ( node ))))
      return NULL;
   return memcpy ( pvNodeData, pvData, sizeof ( node ));
}


extern void pfnFreeListNode ( void *pvData )
/*
   Free a list node
*/
{
   assert ( pvData != NULL );
   free ( pvData );
   pvData = NULL;
}



int main (int argc, char *argv[])
{
   int i;
   node thisNode;
   node * pThisNode = NULL;
   List_t * pList = NULL;

   if ( NULL == ( pList = List_new ( pfnCompareListNodes, pfnCreateListNode, pfnFreeListNode )))
   {
      printf ( "Unable to create list\n" );
      exit ( EXIT_FAILURE );
   }
   for ( i = 0; i < 100 ; ++i )
   {/* fill list with 100 random numbers between 1 and 100 */
      thisNode.value = (rand() % 100) + 1;
      if ( ! List_addAfter ( pList, &thisNode))
      {
         printf ( "Unable to add to list\n" );
         exit ( EXIT_FAILURE );
      }
   }
   /* print list */
   printf ( "\nList prior to sorting:\n" );
   List_moveToHead (pList);
   while ( NULL != ( pThisNode = List_walk ( pList )))
   {
      printf ( "%i%s", pThisNode->value, List_notAtEnd (pList ) ? ", " : "\n\n");
   }
   List_quickSort ( pList );
   /* print list now it has been sorted */
   printf ( "List after sorting:\n" );
   List_moveToHead (pList);
   while ( NULL != ( pThisNode = List_walk ( pList )))
   {
      printf ( "%i%s", pThisNode->value, List_notAtEnd (pList ) ? ", " : "\n\n" );
   }
   /* deduplicate sorted list */
   List_deduplicate ( pList );
   printf ( "List after deduplication (%li nodes removed):\n", 100L - List_numberOfNodes (pList));
   List_moveToHead (pList);
   while ( NULL != ( pThisNode = List_walk ( pList )))
   {
      printf ( "%i%s", pThisNode->value, List_notAtEnd (pList ) ? ", " : "\n\n" );
   }
   /* clean up before closing */
   List_delete( &pList);
   fflush ( NULL );
   return EXIT_SUCCESS;
}


#ifdef __cplusplus
};
#endif


